@@ -342,6 +342,8 @@ mapi_auth_loop (CamelService *service, GError **error)
 	service->url->passwd = NULL;
 
 	while (!authenticated) {
+		GError *mapi_error = NULL;
+
 		if (errbuf) {
 			/* We need to un-cache the password before prompting again */
 			prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
@@ -376,10 +378,14 @@ mapi_auth_loop (CamelService *service, GError **error)
 			}
 		}
 
-		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, service->url->passwd);
+		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, service->url->passwd, &mapi_error);
 		if (!store->priv->conn || !exchange_mapi_connection_connected (store->priv->conn)) {
-			errbuf = g_strdup_printf (_("Unable to authenticate to Exchange MAPI server."));
-
+			if (mapi_error) {
+				errbuf = g_strdup_printf (_("Unable to authenticate to Exchange MAPI server: %s"), mapi_error->message);
+				g_error_free (mapi_error);
+			} else {
+				errbuf = g_strdup (_("Unable to authenticate to Exchange MAPI server"));
+			}
 		} else
 			authenticated = TRUE;
 
@@ -549,6 +555,7 @@ mapi_create_folder(CamelStore *store, const gchar *parent_name, const gchar *fol
 	CamelFolderInfo *root = NULL;
 	gchar *parent_id;
 	mapi_id_t parent_fid, new_folder_id;
+	GError *mapi_error = NULL;
 
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		g_set_error (
@@ -581,7 +588,7 @@ mapi_create_folder(CamelStore *store, const gchar *parent_name, const gchar *fol
 	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	exchange_mapi_util_mapi_id_from_string (parent_id, &parent_fid);
-	new_folder_id = exchange_mapi_connection_create_folder (priv->conn, olFolderInbox, parent_fid, 0, folder_name);
+	new_folder_id = exchange_mapi_connection_create_folder (priv->conn, olFolderInbox, parent_fid, 0, folder_name, &mapi_error);
 	if (new_folder_id != 0) {
 		CamelMapiStoreInfo *si;
 		gchar *fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", new_folder_id);
@@ -589,11 +596,23 @@ mapi_create_folder(CamelStore *store, const gchar *parent_name, const gchar *fol
 		root = mapi_build_folder_info(mapi_store, parent_name, folder_name);
 
 		si = camel_mapi_store_summary_add_from_full(mapi_store->summary, root->full_name, '/', fid, parent_id);
+		si->info.flags |= CAMEL_MAPI_FOLDER_MAIL;
 		camel_store_summary_save((CamelStoreSummary *)mapi_store->summary);
 
 		mapi_update_folder_hash_tables (mapi_store, root->full_name, fid, parent_id);
 
 		camel_store_folder_created (store, root);
+	} else {
+		if (mapi_error) {
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot create folder '%s': %s"), folder_name, mapi_error->message);
+			g_error_free (mapi_error);
+		} else {
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot create folder '%s'"), folder_name);
+		}
 	}
 
 	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -667,7 +686,7 @@ mapi_delete_folder(CamelStore *store, const gchar *folder_name, GError **error)
 
 	folder_id = g_hash_table_lookup (priv->name_hash, folder_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &folder_fid);
-	status = exchange_mapi_connection_remove_folder (priv->conn, folder_fid, 0);
+	status = exchange_mapi_connection_remove_folder (priv->conn, folder_fid, 0, &local_error);
 
 	if (status) {
 		/* Fixme ??  */
@@ -679,6 +698,22 @@ mapi_delete_folder(CamelStore *store, const gchar *folder_name, GError **error)
 		/*g_hash_table_remove (priv->parent_hash, folder_id);*/
 		g_hash_table_remove (priv->id_hash, folder_id);
 		g_hash_table_remove (priv->name_hash, folder_name);
+	} else {
+		success = FALSE;
+
+		if (local_error) {
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot remove folder '%s': %s"),
+				folder_name, local_error->message);
+
+			g_error_free (local_error);
+		} else {
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot remove folder '%s'"),
+				folder_name);
+		}
 	}
 
 	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -817,13 +852,22 @@ mapi_rename_folder(CamelStore *store, const gchar *old_name, const gchar *new_na
 		gchar *folder_id;
 
 		/* renaming in the same folder, thus no MoveFolder necessary */
-		if (!exchange_mapi_connection_rename_folder (priv->conn, old_fid, 0, tmp ? tmp : new_name)) {
-			/*To translators : '%s to %s' is current name of the folder  and
-			new name of the folder.*/
-			g_set_error (
-				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-				_("Cannot rename MAPI folder '%s' to '%s'"),
-				old_name, new_name);
+		if (!exchange_mapi_connection_rename_folder (priv->conn, old_fid, 0, tmp ? tmp : new_name, &local_error)) {
+			if (local_error) {
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+					/* Translators: '%s to %s' is current name of the folder and new name of the folder.
+					   The last '%s' is a detailed error message. */
+					_("Cannot rename MAPI folder '%s' to '%s': %s"),
+					old_name, new_name, local_error->message);
+				g_error_free (local_error);
+			} else {
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+					/* Translators: '%s to %s' is current name of the folder and new name of the folder. */
+					_("Cannot rename MAPI folder '%s' to '%s'"),
+					old_name, new_name);
+			}
 
 			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			g_free (old_parent);
@@ -872,12 +916,20 @@ mapi_rename_folder(CamelStore *store, const gchar *old_name, const gchar *new_na
 		} else if (!old_parent_fid_str || !new_parent_fid_str ||
 			   !exchange_mapi_util_mapi_id_from_string (old_parent_fid_str, &old_parent_fid) ||
 			   !exchange_mapi_util_mapi_id_from_string (new_parent_fid_str, &new_parent_fid) ||
-			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, 0, new_parent_fid, 0, tmp)) {
+			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, 0, new_parent_fid, 0, tmp, &local_error)) {
 			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-			g_set_error (
-				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
-				_("Cannot rename MAPI folder '%s' to '%s'"),
-				old_name, new_name);
+			if (local_error) {
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+					_("Cannot rename MAPI folder '%s' to '%s': %s"),
+					old_name, new_name, local_error->message);
+				g_error_free (local_error);
+			} else {
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+					_("Cannot rename MAPI folder '%s' to '%s'"),
+					old_name, new_name);
+			}
 			g_free (old_parent);
 			g_free (new_parent);
 			return FALSE;
@@ -1139,8 +1191,8 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 			camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 			continue;
 		}
-		
-		if (!subscription_list && !(si->flags & CAMEL_MAPI_FOLDER_MAIL) && si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED && 
+
+		if (!subscription_list && !(si->flags & CAMEL_MAPI_FOLDER_MAIL) && si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED &&
 												si->flags & CAMEL_MAPI_FOLDER_PUBLIC)
 			continue;
 		if (!strcmp(name, camel_mapi_store_info_full_name (mapi_store->summary, si))
@@ -1285,7 +1337,7 @@ camel_mapi_store_connected (CamelMapiStore *store, GError **error)
 	    && camel_service_connect ((CamelService *)store, error));
 }
 
-static void 
+static void
 mapi_update_hash_table_type (CamelMapiStore *store, const gchar *full_name, guint *folder_type)
 {
 	CamelMapiStorePrivate  *priv = store->priv;
@@ -1335,12 +1387,12 @@ mapi_folders_update_hash_tables_from_cache (CamelMapiStore *store)
 
 /* static const gchar * */
 
-guint 
-mapi_folders_hash_table_type_lookup (CamelMapiStore *store, const gchar *name) 
+guint
+mapi_folders_hash_table_type_lookup (CamelMapiStore *store, const gchar *name)
 {
 	CamelMapiStorePrivate  *priv = store->priv;
 	guint *folder_type = g_hash_table_lookup (priv->container_hash, name);
-	
+
 	return *folder_type;
 }
 
@@ -1432,6 +1484,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	guint32 count, i;
 	CamelStoreInfo *si = NULL;
 	GHashTable *old_cache_folders;
+	GError *err = NULL;
 
 	if (!camel_mapi_store_connected (store, NULL)) {
 		g_set_error (
@@ -1441,10 +1494,12 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 		return FALSE;
 	}
 
-	status = exchange_mapi_connection_get_folders_list (priv->conn, &folder_list);
+	status = exchange_mapi_connection_get_folders_list (priv->conn, &folder_list, &err);
 
 	if (!status) {
-		g_warning ("Could not get folder list..\n");
+		g_warning ("Could not get folder list (%s)\n", err ? err->message : "Unknown error");
+		if (err)
+			g_error_free (err);
 		return TRUE;
 	}
 
@@ -1467,10 +1522,15 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 
 	subscription_list = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST);
 	if (subscription_list) {
+		GError *err = NULL;
+
 		/*Consult the name <-> fid hash table for a FID.*/
-		status = exchange_mapi_connection_get_pf_folders_list (priv->conn, &folder_list);
+		status = exchange_mapi_connection_get_pf_folders_list (priv->conn, &folder_list, &err);
 		if (!status)
-			g_warning ("Could not get Public folder list..\n");
+			g_warning ("Could not get Public folder list (%s)\n", err ? err->message : "Unknown error");
+
+		if (err)
+			g_error_free (err);
 	}
 
 	temp_list = folder_list;
@@ -1533,33 +1593,33 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, NULL);
 			info->flags |= CAMEL_MAPI_FOLDER_MAIL;
 			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
-			
+
 			if (!mapi_si) {
 				gchar *fid, *pfid = NULL;
-	
+
 				fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
 						       exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
 				pfid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
 							exchange_mapi_folder_get_parent_id((ExchangeMAPIFolder *)(folder_list->data)));
-	
+
 				mapi_si = camel_mapi_store_summary_add_from_full (store->summary, info->full_name, '/', fid, pfid);
 				g_free (fid);
 				g_free (pfid);
 				if (mapi_si == NULL)
 					continue;
-	
+
 				camel_store_summary_info_ref ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
-	
+
 				if (!subscription_list) {
 					camel_store_folder_created (CAMEL_STORE (store), info);
 					camel_store_folder_subscribed (CAMEL_STORE (store), info);
 				}
 			}
-		
+
 			mapi_si->info.flags |= info->flags;
 			mapi_si->info.total = info->total;
 			mapi_si->info.unread = info->unread;
-			
+
 			camel_store_summary_info_free ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
 			camel_folder_info_free (info);
 		} else if (folder->category == MAPI_FAVOURITE_FOLDER) {
@@ -1580,10 +1640,10 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			camel_store_summary_info_ref ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
 			mapi_si->info.flags |= info->flags;
 			camel_store_summary_info_free ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
-			camel_folder_info_free (info);	
+			camel_folder_info_free (info);
 		}
 	}
-	
+
 	/* Weed out deleted folders */
 	g_hash_table_foreach (old_cache_folders, (GHFunc) remove_path_from_store_summary, store);
 	g_hash_table_destroy (old_cache_folders);
@@ -1705,13 +1765,12 @@ static gboolean
 mapi_subscribe_folder(CamelStore *store, const gchar *folder_name, GError **error)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-	guint folder_type = mapi_folders_hash_table_type_lookup(mapi_store, folder_name);
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si = NULL;
 	const gchar *parent_name = NULL, *use_folder_name = folder_name, *fid = NULL;
 	gboolean favourites = FALSE;
 	/* TODO : exchange_mapi_add_to_favorites (); */
-	
+
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 
 	if (g_str_has_prefix (folder_name, DISPLAY_NAME_ALL_PUBLIC_FOLDERS) ) {
@@ -1738,8 +1797,8 @@ mapi_subscribe_folder(CamelStore *store, const gchar *folder_name, GError **erro
 		}
 	}
 
-	if (folder_type == MAPI_FOLDER_TYPE_MAIL || folder_type == MAPI_FOLDER_TYPE_UNKNOWN) {
-		fi = mapi_build_folder_info (mapi_store, parent_name, use_folder_name);		
+	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
+		fi = mapi_build_folder_info (mapi_store, parent_name, use_folder_name);
 		if (favourites) {
 			CamelURL *url;
 			url = camel_url_new (mapi_store->priv->base_url, NULL);
@@ -1757,9 +1816,10 @@ mapi_subscribe_folder(CamelStore *store, const gchar *folder_name, GError **erro
 		fi->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
 		camel_store_folder_subscribed (store, fi);
 		camel_folder_info_free (fi);
-	} else
+	} else {
+		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, folder_name);
 		exchange_mapi_add_esource (CAMEL_SERVICE(mapi_store)->url, use_folder_name, fid, folder_type);
-	
+	}
 	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	return TRUE;
 }
@@ -1790,7 +1850,6 @@ mapi_unsubscribe_folder(CamelStore *store, const gchar *folder_name, GError **er
 
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelURL *url = CAMEL_SERVICE (mapi_store)->url;
-	guint folder_type = mapi_folders_hash_table_type_lookup(mapi_store, folder_name);
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
 	if (si) {
@@ -1814,11 +1873,12 @@ mapi_unsubscribe_folder(CamelStore *store, const gchar *folder_name, GError **er
 		fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
 		camel_store_folder_unsubscribed (store, fi);
 		camel_folder_info_free (fi);
-	} else 
+	} else {
+		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, folder_name);
 		exchange_mapi_remove_esource(url, folder_name, fid, folder_type);
-	
+	}
 	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
-		
+
 	return TRUE;
 }
 
